@@ -18,26 +18,24 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <glib/gi18n.h>
 
-#include "gossip-contact.h"
-#include "gossip-message.h"
 #include "gossip-chatroom-provider.h"
+#include "gossip-message.h"
+
 #include "libgossip-marshal.h"
 
 static void chatroom_provider_base_init (gpointer g_class);
 
 enum {
 	CHATROOM_JOINED,
+	CHATROOM_KICKED,
 	CHATROOM_NEW_MESSAGE,
 	CHATROOM_NEW_EVENT,
 	CHATROOM_TOPIC_CHANGED,
-	CHATROOM_CONTACT_JOINED,
-	CHATROOM_CONTACT_LEFT,
-	CHATROOM_CONTACT_PRESENCE_UPDATED,
-	CHATROOM_CONTACT_UPDATED,
+	CHATROOM_ERROR,
 	LAST_SIGNAL
 };
 
@@ -88,6 +86,16 @@ chatroom_provider_base_init (gpointer g_class)
 				      G_TYPE_NONE,
 				      1, G_TYPE_INT);
 
+		signals[CHATROOM_KICKED] =
+			g_signal_new ("chatroom-kicked",
+				      G_TYPE_FROM_CLASS (g_class),
+				      G_SIGNAL_RUN_LAST,
+				      0,
+				      NULL, NULL,
+				      libgossip_marshal_VOID__INT,
+				      G_TYPE_NONE,
+				      1, G_TYPE_INT);
+
 		signals[CHATROOM_NEW_MESSAGE] =
 			g_signal_new ("chatroom-new-message",
 				      G_TYPE_FROM_CLASS (g_class),
@@ -115,47 +123,16 @@ chatroom_provider_base_init (gpointer g_class)
 				      libgossip_marshal_VOID__INT_OBJECT_STRING,
 				      G_TYPE_NONE,
 				      3, G_TYPE_INT, GOSSIP_TYPE_CONTACT, G_TYPE_STRING);
-
-		signals[CHATROOM_CONTACT_JOINED] =
-			g_signal_new ("chatroom-contact-joined",
+		signals[CHATROOM_ERROR] =
+			g_signal_new ("chatroom-error",
 				      G_TYPE_FROM_CLASS (g_class),
 				      G_SIGNAL_RUN_LAST,
 				      0,
 				      NULL, NULL,
-				      libgossip_marshal_VOID__INT_OBJECT,
+				      libgossip_marshal_VOID__INT_INT,
 				      G_TYPE_NONE,
-				      2, G_TYPE_INT, GOSSIP_TYPE_CONTACT);
-
-		signals[CHATROOM_CONTACT_LEFT] =
-			g_signal_new ("chatroom-contact-left",
-				      G_TYPE_FROM_CLASS (g_class),
-				      G_SIGNAL_RUN_LAST,
-				      0,
-				      NULL, NULL,
-				      libgossip_marshal_VOID__INT_OBJECT,
-				      G_TYPE_NONE,
-				      2, G_TYPE_INT, GOSSIP_TYPE_CONTACT);
-
-		signals[CHATROOM_CONTACT_PRESENCE_UPDATED] =
-			g_signal_new ("chatroom-contact-presence-updated",
-				      G_TYPE_FROM_CLASS (g_class),
-				      G_SIGNAL_RUN_LAST,
-				      0,
-				      NULL, NULL,
-				      libgossip_marshal_VOID__INT_OBJECT,
-				      G_TYPE_NONE,
-				      2, G_TYPE_INT, GOSSIP_TYPE_CONTACT);
-
-		signals[CHATROOM_CONTACT_UPDATED] =
-			g_signal_new ("chatroom-contact-updated",
-				      G_TYPE_FROM_CLASS (g_class),
-				      G_SIGNAL_RUN_LAST,
-				      0,
-				      NULL, NULL,
-				      libgossip_marshal_VOID__INT_OBJECT,
-				      G_TYPE_NONE,
-				      2, G_TYPE_INT, GOSSIP_TYPE_CONTACT);
-
+				      2, G_TYPE_INT, G_TYPE_INT);
+	
 		initialized = TRUE;
 	}
 }
@@ -254,6 +231,39 @@ gossip_chatroom_provider_leave (GossipChatroomProvider *provider,
 	}
 }
 
+void
+gossip_chatroom_provider_kick (GossipChatroomProvider *provider,
+			       GossipChatroomId        id,
+			       GossipContact          *contact,
+			       const gchar            *reason)
+{
+	g_return_if_fail (GOSSIP_IS_CHATROOM_PROVIDER (provider));
+	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
+	g_return_if_fail (id > 0);
+
+	if (GOSSIP_CHATROOM_PROVIDER_GET_IFACE (provider)->kick) {
+		GOSSIP_CHATROOM_PROVIDER_GET_IFACE (provider)->kick (provider,
+								     id,
+								     contact,
+								     reason);
+	}
+}
+
+GSList *
+gossip_chatroom_provider_get_contacts (GossipChatroomProvider *provider,
+				       GossipChatroomId        id)
+{
+	g_return_val_if_fail (GOSSIP_IS_CHATROOM_PROVIDER (provider), NULL);
+	g_return_val_if_fail (id > 0, NULL);
+
+	if (GOSSIP_CHATROOM_PROVIDER_GET_IFACE (provider)->get_contacts) {
+		return GOSSIP_CHATROOM_PROVIDER_GET_IFACE (provider)->get_contacts (provider,
+										    id);
+	}
+
+	return NULL;
+}
+
 GossipChatroom *
 gossip_chatroom_provider_find_by_id (GossipChatroomProvider *provider,
 				     GossipChatroomId        id)
@@ -366,38 +376,59 @@ gossip_chatroom_provider_browse_rooms (GossipChatroomProvider *provider,
 }
 
 const gchar *
-gossip_chatroom_provider_join_result_as_str (GossipChatroomJoinResult result)
+gossip_chatroom_provider_error_to_string (GossipChatroomError error)
 {
-	const gchar *last_error = NULL;
+	const gchar *str = NULL;
 
-	switch (result) {
-	case GOSSIP_CHATROOM_JOIN_NICK_IN_USE:
-		last_error = _("The nickname you have chosen is already in use.");
+	switch (error) {
+	case GOSSIP_CHATROOM_ERROR_PASSWORD_INVALID_OR_MISSING:
+		str = _("The chat room you tried to join requires a password. "
+			"You either failed to supply a password or the password you tried was incorrect.");
 		break;
 
-	case GOSSIP_CHATROOM_JOIN_NEED_PASSWORD:
-		last_error = _("The chat room you tried to join requires a password.");
+	case GOSSIP_CHATROOM_ERROR_USER_BANNED:
+		str = _("You have been banned from this chatroom.");
 		break;
 
-	case GOSSIP_CHATROOM_JOIN_TIMED_OUT:
-		last_error = _("The remote conference server did not respond in a sensible time.");
+	case GOSSIP_CHATROOM_ERROR_ROOM_NOT_FOUND:
+		str = _("The conference room you tried to join could not be found.");
 		break;
 
-	case GOSSIP_CHATROOM_JOIN_UNKNOWN_HOST:
-		last_error = _("The conference server you tried to join could not be found.");
+	case GOSSIP_CHATROOM_ERROR_ROOM_CREATION_RESTRICTED:
+		str = _("Chatroom creation is restricted on this server.");
 		break;
 
-	case GOSSIP_CHATROOM_JOIN_UNKNOWN_ERROR:
-		last_error = _("An unknown error occurred, check your details are correct.");
+	case GOSSIP_CHATROOM_ERROR_USE_RESERVED_ROOM_NICK:
+		str = _("Chatroom reserved nick names must be used on this server.");
 		break;
 
-	case GOSSIP_CHATROOM_JOIN_CANCELED:
-		last_error = _("Joining the chatroom was canceled.");
+	case GOSSIP_CHATROOM_ERROR_NOT_ON_MEMBERS_LIST:
+		str = _("You are not on the chatroom's members list.");
+		break;
+
+	case GOSSIP_CHATROOM_ERROR_NICK_IN_USE:
+		str = _("The nickname you have chosen is already in use.");
+		break;
+
+	case GOSSIP_CHATROOM_ERROR_MAXIMUM_USERS_REACHED:
+		str = _("The maximum number of users for this chatroom has been reached.");
+		break;
+
+	case GOSSIP_CHATROOM_ERROR_TIMED_OUT:
+		str = _("The remote conference server did not respond in a sensible time.");
+		break;
+
+	case GOSSIP_CHATROOM_ERROR_UNKNOWN:
+		str = _("An unknown error occurred, check your details are correct.");
+		break;
+
+	case GOSSIP_CHATROOM_ERROR_CANCELED:
+		str = _("Joining the chatroom was canceled.");
 		break;
 
 	default:
 		break;
 	}
 
-	return last_error;
+	return str;
 }
