@@ -24,8 +24,11 @@
 #include <glib-object.h>
 
 #include "gossip-chatroom.h"
+#include "gossip-chatroom-invite.h"
 #include "gossip-contact.h"
 
+G_BEGIN_DECLS
+
 #define GOSSIP_TYPE_CHATROOM_PROVIDER           (gossip_chatroom_provider_get_type ())
 #define GOSSIP_CHATROOM_PROVIDER(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOSSIP_TYPE_CHATROOM_PROVIDER, GossipChatroomProvider))
 #define GOSSIP_IS_CHATROOM_PROVIDER(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GOSSIP_TYPE_CHATROOM_PROVIDER))
@@ -35,20 +38,34 @@ typedef struct _GossipChatroomProvider      GossipChatroomProvider;
 typedef struct _GossipChatroomProviderIface GossipChatroomProviderIface;
 
 typedef enum {
-	GOSSIP_CHATROOM_JOIN_OK,
-	GOSSIP_CHATROOM_JOIN_NICK_IN_USE,
-	GOSSIP_CHATROOM_JOIN_NEED_PASSWORD,
-	GOSSIP_CHATROOM_JOIN_ALREADY_OPEN,
-	GOSSIP_CHATROOM_JOIN_TIMED_OUT,
-	GOSSIP_CHATROOM_JOIN_UNKNOWN_HOST,
-	GOSSIP_CHATROOM_JOIN_UNKNOWN_ERROR,
-	GOSSIP_CHATROOM_JOIN_CANCELED
-} GossipChatroomJoinResult;
+	GOSSIP_CHATROOM_ERROR_NONE,
 
-typedef void (*GossipChatroomJoinCb) (GossipChatroomProvider   *provider,
-				      GossipChatroomJoinResult  result,
-				      gint                      id,
-				      gpointer                  user_data);
+	/* MUC errors */
+	GOSSIP_CHATROOM_ERROR_PASSWORD_INVALID_OR_MISSING,
+	GOSSIP_CHATROOM_ERROR_USER_BANNED,
+	GOSSIP_CHATROOM_ERROR_ROOM_NOT_FOUND,
+	GOSSIP_CHATROOM_ERROR_ROOM_CREATION_RESTRICTED,
+	GOSSIP_CHATROOM_ERROR_USE_RESERVED_ROOM_NICK,
+	GOSSIP_CHATROOM_ERROR_NOT_ON_MEMBERS_LIST,
+	GOSSIP_CHATROOM_ERROR_NICK_IN_USE,
+	GOSSIP_CHATROOM_ERROR_MAXIMUM_USERS_REACHED,
+
+	/* Internal errors */
+	GOSSIP_CHATROOM_ERROR_ALREADY_OPEN,
+	GOSSIP_CHATROOM_ERROR_TIMED_OUT,
+	GOSSIP_CHATROOM_ERROR_CANCELED,
+	GOSSIP_CHATROOM_ERROR_UNKNOWN
+} GossipChatroomError;
+
+typedef void (*GossipChatroomJoinCb)   (GossipChatroomProvider   *provider,
+					GossipChatroomId          id,
+					GossipChatroomError       error,
+					gpointer                  user_data);
+typedef void (*GossipChatroomBrowseCb) (GossipChatroomProvider   *provider,
+					const gchar              *server,
+					GList                    *rooms,
+					GError                   *error,
+					gpointer                  user_data);
 
 struct _GossipChatroomProviderIface {
 	GTypeInterface g_iface;
@@ -71,8 +88,16 @@ struct _GossipChatroomProviderIface {
 					     const gchar            *new_nick);
 	void             (*leave)           (GossipChatroomProvider *provider,
 					     GossipChatroomId        id);
-	GossipChatroom * (*find)            (GossipChatroomProvider *provider,
+	void             (*kick)            (GossipChatroomProvider *provider,
+					     GossipChatroomId        id,
+					     GossipContact          *contact,
+					     const gchar            *reason);
+	GSList *         (*get_contacts)    (GossipChatroomProvider *provider,
 					     GossipChatroomId        id);
+	GossipChatroom * (*find_by_id)      (GossipChatroomProvider *provider,
+					     GossipChatroomId        id);
+	GossipChatroom * (*find)            (GossipChatroomProvider *provider,
+					     GossipChatroom         *chatroom);
 	void             (*invite)          (GossipChatroomProvider *provider,
 					     GossipChatroomId        id,
 					     GossipContact          *contact,
@@ -85,6 +110,10 @@ struct _GossipChatroomProviderIface {
 					     GossipChatroomInvite   *invite,
 					     const gchar            *reason);
 	GList *          (*get_rooms)       (GossipChatroomProvider *provider);
+	void             (*browse_rooms)    (GossipChatroomProvider *provider,
+					     const gchar            *server,
+					     GossipChatroomBrowseCb  callback,
+					     gpointer                user_data);
 };
 
 GType        gossip_chatroom_provider_get_type           (void) G_GNUC_CONST;
@@ -107,9 +136,18 @@ void         gossip_chatroom_provider_change_nick        (GossipChatroomProvider
 							  const gchar            *new_nick);
 void         gossip_chatroom_provider_leave              (GossipChatroomProvider *provider,
 							  GossipChatroomId        id);
+void         gossip_chatroom_provider_kick               (GossipChatroomProvider *provider,
+							  GossipChatroomId        id,
+							  GossipContact          *contact,
+							  const gchar            *reason);
+GSList *     gossip_chatroom_provider_get_contacts       (GossipChatroomProvider *provider,
+							  GossipChatroomId        id);
+GossipChatroom *
+	     gossip_chatroom_provider_find_by_id         (GossipChatroomProvider *provider,
+							  GossipChatroomId        id);
 GossipChatroom *
 	     gossip_chatroom_provider_find               (GossipChatroomProvider *provider,
-							  GossipChatroomId        id);
+							  GossipChatroom         *chatroom);
 void         gossip_chatroom_provider_invite             (GossipChatroomProvider *provider,
 							  GossipChatroomId        id,
 							  GossipContact          *contact,
@@ -123,8 +161,13 @@ void         gossip_chatroom_provider_invite_decline     (GossipChatroomProvider
 							  const gchar            *reason);
 
 GList *      gossip_chatroom_provider_get_rooms          (GossipChatroomProvider *provider);
+void         gossip_chatroom_provider_browse_rooms       (GossipChatroomProvider *provider,
+							  const gchar            *server,
+							  GossipChatroomBrowseCb  callback,
+							  gpointer                user_data);
 
-const gchar *gossip_chatroom_provider_join_result_as_str (GossipChatroomJoinResult result);
+const gchar *gossip_chatroom_provider_error_to_string    (GossipChatroomError     error);
 
+G_END_DECLS
 
 #endif /* __GOSSIP_CHATROOM_PROVIDER_H__ */
