@@ -16,8 +16,6 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
- *
- * Authors: Martyn Russell <martyn@imendio.com>
  */
 
 #include "config.h"
@@ -36,63 +34,61 @@
 typedef struct _GossipChatroomPriv GossipChatroomPriv;
 
 struct _GossipChatroomPriv {
-	GossipAccount        *account;
+	GossipAccount         *account;
 
-	GossipChatroomType    type;
+	GossipChatroomId       id;
+	gchar                 *id_str;
 
-	GossipChatroomId      id;
+	gchar                 *name;
+	gchar                 *description;
+	gchar                 *subject;
+	gchar                 *nick;
+	gchar                 *server;
+	gchar                 *room;
+	gchar                 *password;
+	gboolean               auto_connect;
+	gboolean               favourite;
 
-	gchar                *id_str;
+	GossipChatroomFeature  features;
+	GossipChatroomStatus   status;
 
-	gchar                *name;
-	gchar                *nick;
-	gchar                *server;
-	gchar                *room;
-	gchar                *password;
-	gboolean              auto_connect;
-	gboolean              favourite;
+	guint                  occupants;
 
-	GossipChatroomStatus  status;
-	gchar                *last_error;
+	gchar                 *last_error;
 
-	GHashTable           *contacts;
+	GHashTable            *contacts;
 };
 
-struct _GossipChatroomInvite {
-	guint          ref_count;
-
-	GossipContact *invitor;
-	gchar         *id;
-	gchar         *reason;
-};
-
-static void chatroom_class_init   (GossipChatroomClass *class);
-static void chatroom_init         (GossipChatroom      *chatroom);
-static void chatroom_finalize     (GObject             *object);
-static void chatroom_get_property (GObject             *object,
-				   guint                param_id,
-				   GValue              *value,
-				   GParamSpec          *pspec);
-static void chatroom_set_property (GObject             *object,
-				   guint                param_id,
-				   const GValue        *value,
-				   GParamSpec          *pspec);
+static void gossip_chatroom_class_init (GossipChatroomClass *klass);
+static void gossip_chatroom_init       (GossipChatroom      *chatroom);
+static void chatroom_finalize          (GObject             *object);
+static void chatroom_get_property      (GObject             *object,
+					guint                param_id,
+					GValue              *value,
+					GParamSpec          *pspec);
+static void chatroom_set_property      (GObject             *object,
+					guint                param_id,
+					const GValue        *value,
+					GParamSpec          *pspec);
 
 enum {
 	PROP_0,
+	PROP_ACCOUNT,
 	PROP_ID,
 	PROP_ID_STR,
-	PROP_TYPE,
 	PROP_NAME,
+	PROP_DESCRIPTION,
+	PROP_SUBJECT,
 	PROP_NICK,
 	PROP_SERVER,
 	PROP_ROOM,
 	PROP_PASSWORD,
 	PROP_AUTO_CONNECT,
 	PROP_FAVOURITE,
+	PROP_FEATURES,
 	PROP_STATUS,
+	PROP_OCCUPANTS,
 	PROP_LAST_ERROR,
-	PROP_ACCOUNT,
 };
 
 enum {
@@ -102,66 +98,21 @@ enum {
 	LAST_SIGNAL
 };
 
-static guint    signals[LAST_SIGNAL] = {0};
+static guint signals[LAST_SIGNAL] = {0};
 
-static gpointer parent_class = NULL;
-
-GType
-gossip_chatroom_get_gtype (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		static const GTypeInfo info = {
-			sizeof (GossipChatroomClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			(GClassInitFunc) chatroom_class_init,
-			NULL, /* class_finalize */
-			NULL, /* class_data */
-			sizeof (GossipChatroom),
-			0,    /* n_preallocs */
-			(GInstanceInitFunc) chatroom_init
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "GossipChatroom",
-					       &info, 0);
-	}
-
-	return type;
-}
-
-GType
-gossip_chatroom_invite_get_gtype (void)
-{
-	static GType type_id = 0;
-
-	if (!type_id) {
-		type_id = g_boxed_type_register_static ("GossipChatroomInvite",
-							(GBoxedCopyFunc) gossip_chatroom_invite_ref,
-							(GBoxedFreeFunc) gossip_chatroom_invite_unref);
-	}
-
-	return type_id;
-}
+G_DEFINE_TYPE (GossipChatroom, gossip_chatroom, G_TYPE_OBJECT);
 
 static void
-chatroom_class_init (GossipChatroomClass *class)
+gossip_chatroom_class_init (GossipChatroomClass *klass)
 {
 	GObjectClass *object_class;
 
-	object_class = G_OBJECT_CLASS (class);
-	parent_class = g_type_class_peek_parent (class);
+	object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize     = chatroom_finalize;
 	object_class->get_property = chatroom_get_property;
 	object_class->set_property = chatroom_set_property;
 
-	/*
-	 * properties
-	 */
-
 	g_object_class_install_property (object_class,
 					 PROP_ID,
 					 g_param_spec_int ("id",
@@ -180,22 +131,28 @@ chatroom_class_init (GossipChatroomClass *class)
 							      NULL,
 							      G_PARAM_READABLE));
 
-	g_object_class_install_property (object_class,
-					 PROP_TYPE,
-					 g_param_spec_int ("type",
-							   "Chatroom Type",
-							   "The chatroom type, e.g. normal",
-							   G_MININT,
-							   G_MAXINT,
-							   GOSSIP_CHATROOM_TYPE_NORMAL,
-							   G_PARAM_READWRITE));
-
 	g_object_class_install_property (object_class,
 					 PROP_NAME,
 					 g_param_spec_string ("name",
 							      "Chatroom Name",
-							      "What you call this chatroom",
-							      "Default",
+							      "The friendly name for this chatroom",
+							      "",
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class,
+					 PROP_DESCRIPTION,
+					 g_param_spec_string ("description",
+							      "Chatroom Description",
+							      "A more detailed description of this chatroom",
+							      "",
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class,
+					 PROP_SUBJECT,
+					 g_param_spec_string ("subject",
+							      "Chatroom Subject",
+							      "The conversation topic",
+							      "Random Crap",
 							      G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
@@ -246,6 +203,16 @@ chatroom_class_init (GossipChatroomClass *class)
 							       FALSE,
 							       G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_FEATURES,
+					 g_param_spec_int ("features",
+							   "Chatroom Features",
+							   "Features of the room, hidden, passworded, etc",
+							   G_MININT,
+							   G_MAXINT,
+							   0,
+							   G_PARAM_READWRITE));
+
 	g_object_class_install_property (object_class,
 					 PROP_STATUS,
 					 g_param_spec_int ("status",
@@ -256,6 +223,16 @@ chatroom_class_init (GossipChatroomClass *class)
 							   GOSSIP_CHATROOM_STATUS_UNKNOWN,
 							   G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_OCCUPANTS,
+					 g_param_spec_uint ("occupants",
+							   "Chatroom Occupant Count",
+							   "The number of estimated occupants in the room",
+							   0,
+							   G_MAXUINT,
+							   0,
+							   G_PARAM_READWRITE));
+
 	g_object_class_install_property (object_class,
 					 PROP_LAST_ERROR,
 					 g_param_spec_string ("last_error",
@@ -272,10 +249,6 @@ chatroom_class_init (GossipChatroomClass *class)
 							      GOSSIP_TYPE_ACCOUNT,
 							      G_PARAM_READWRITE));
 
-	/*
-	 * signals
-	 */
-
 	signals[CONTACT_JOINED] =
 		g_signal_new ("contact-joined",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -308,7 +281,7 @@ chatroom_class_init (GossipChatroomClass *class)
 }
 
 static void
-chatroom_init (GossipChatroom *chatroom)
+gossip_chatroom_init (GossipChatroom *chatroom)
 {
 	GossipChatroomPriv *priv;
 	static guint        id = 1;
@@ -316,23 +289,12 @@ chatroom_init (GossipChatroom *chatroom)
 	priv = GET_PRIV (chatroom);
 
 	priv->id           = id++;
-	priv->id_str       = NULL;
-
-	priv->type         = 0;
-
-	priv->name         = NULL;
-	priv->nick         = NULL;
-	priv->server       = NULL;
-	priv->room         = NULL;
-	priv->password     = NULL;
 
 	priv->auto_connect = FALSE;
 	priv->favourite    = FALSE;
 
 	priv->status       = GOSSIP_CHATROOM_STATUS_INACTIVE;
-	priv->last_error   = NULL;
 
-	priv->account      = NULL;
 	priv->contacts     = g_hash_table_new_full (gossip_contact_hash,
 						    gossip_contact_equal,
 						    (GDestroyNotify) g_object_unref,
@@ -362,7 +324,7 @@ chatroom_finalize (GObject *object)
 
 	g_hash_table_destroy (priv->contacts);
 
-	(G_OBJECT_CLASS (parent_class)->finalize) (object);
+	(G_OBJECT_CLASS (gossip_chatroom_parent_class)->finalize) (object);
 }
 
 static void
@@ -376,18 +338,24 @@ chatroom_get_property (GObject    *object,
 	priv = GET_PRIV (object);
 
 	switch (param_id) {
+	case PROP_ACCOUNT:
+		g_value_set_object (value, priv->account);
+		break;
 	case PROP_ID:
 		g_value_set_int (value, priv->id);
 		break;
 	case PROP_ID_STR:
 		g_value_set_string (value, priv->id_str);
 		break;
-	case PROP_TYPE:
-		g_value_set_int (value, priv->type);
-		break;
 	case PROP_NAME:
 		g_value_set_string (value, priv->name);
 		break;
+	case PROP_DESCRIPTION:
+		g_value_set_string (value, priv->description);
+		break;
+	case PROP_SUBJECT:
+		g_value_set_string (value, priv->subject);
+		break;
 	case PROP_NICK:
 		g_value_set_string (value, priv->nick);
 		break;
@@ -406,15 +374,18 @@ chatroom_get_property (GObject    *object,
 	case PROP_FAVOURITE:
 		g_value_set_boolean (value, priv->favourite);
 		break;
+	case PROP_FEATURES:
+		g_value_set_int (value, priv->features);
+		break;
 	case PROP_STATUS:
 		g_value_set_int (value, priv->status);
 		break;
+	case PROP_OCCUPANTS:
+		g_value_set_uint (value, priv->occupants);
+		break;
 	case PROP_LAST_ERROR:
 		g_value_set_string (value, priv->last_error);
 		break;
-	case PROP_ACCOUNT:
-		g_value_set_object (value, priv->account);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -432,13 +403,22 @@ chatroom_set_property (GObject      *object,
 	priv = GET_PRIV (object);
 
 	switch (param_id) {
-	case PROP_TYPE:
-		priv->type = g_value_get_int (value);
+	case PROP_ACCOUNT:
+		gossip_chatroom_set_account (GOSSIP_CHATROOM (object),
+					     g_value_get_object (value));
 		break;
 	case PROP_NAME:
 		gossip_chatroom_set_name (GOSSIP_CHATROOM (object),
 					  g_value_get_string (value));
 		break;
+	case PROP_DESCRIPTION:
+		gossip_chatroom_set_description (GOSSIP_CHATROOM (object),
+						 g_value_get_string (value));
+		break;
+	case PROP_SUBJECT:
+		gossip_chatroom_set_subject (GOSSIP_CHATROOM (object),
+					     g_value_get_string (value));
+		break;
 	case PROP_NICK:
 		gossip_chatroom_set_nick (GOSSIP_CHATROOM (object),
 				       g_value_get_string (value));
@@ -463,24 +443,44 @@ chatroom_set_property (GObject      *object,
 		gossip_chatroom_set_favourite (GOSSIP_CHATROOM (object),
 					       g_value_get_boolean (value));
 		break;
+	case PROP_FEATURES:
+		gossip_chatroom_set_features (GOSSIP_CHATROOM (object),
+					      g_value_get_int (value));
+		break;
 	case PROP_STATUS:
 		gossip_chatroom_set_status (GOSSIP_CHATROOM (object),
 					    g_value_get_int (value));
 		break;
+	case PROP_OCCUPANTS:
+		gossip_chatroom_set_occupants (GOSSIP_CHATROOM (object),
+					       g_value_get_uint (value));
+		break;
 	case PROP_LAST_ERROR:
 		gossip_chatroom_set_last_error (GOSSIP_CHATROOM (object),
 						g_value_get_string (value));
 		break;
-	case PROP_ACCOUNT:
-		gossip_chatroom_set_account (GOSSIP_CHATROOM (object),
-					     g_value_get_object (value));
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
 	};
 }
 
+GossipAccount *
+gossip_chatroom_get_account (GossipChatroom *chatroom)
+{
+	GossipChatroomPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), NULL);
+
+	priv = GET_PRIV (chatroom);
+
+	if (!priv->account) {
+		return NULL;
+	}
+
+	return priv->account;
+}
+
 GossipChatroomId
 gossip_chatroom_get_id (GossipChatroom *chatroom)
 {
@@ -492,17 +492,6 @@ gossip_chatroom_get_id (GossipChatroom *chatroom)
 	return priv->id;
 }
 
-GossipChatroomType
-gossip_chatroom_get_type (GossipChatroom *chatroom)
-{
-	GossipChatroomPriv *priv;
-
-	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), 0);
-
-	priv = GET_PRIV (chatroom);
-	return priv->type;
-}
-
 const gchar *
 gossip_chatroom_get_id_str (GossipChatroom *chatroom)
 {
@@ -530,6 +519,28 @@ gossip_chatroom_get_name (GossipChatroom *chatroom)
 	return priv->name;
 }
 
+const gchar *
+gossip_chatroom_get_description (GossipChatroom *chatroom)
+{
+	GossipChatroomPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), NULL);
+
+	priv = GET_PRIV (chatroom);
+	return priv->description;
+}
+
+const gchar *
+gossip_chatroom_get_subject (GossipChatroom *chatroom)
+{
+	GossipChatroomPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), NULL);
+
+	priv = GET_PRIV (chatroom);
+	return priv->subject;
+}
+
 const gchar *
 gossip_chatroom_get_nick (GossipChatroom *chatroom)
 {
@@ -586,16 +597,29 @@ gossip_chatroom_get_auto_connect (GossipChatroom *chatroom)
 }
 
 gboolean
-gossip_chatroom_get_is_favourite (GossipChatroom *chatroom)
+gossip_chatroom_get_favourite (GossipChatroom *chatroom)
 {
 	GossipChatroomPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), FALSE);
 
 	priv = GET_PRIV (chatroom);
+
 	return priv->favourite;
 }
 
+GossipChatroomFeature
+gossip_chatroom_get_features (GossipChatroom *chatroom)
+{
+	GossipChatroomPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), 0);
+
+	priv = GET_PRIV (chatroom);
+
+	return priv->features;
+}
+
 GossipChatroomStatus
 gossip_chatroom_get_status (GossipChatroom *chatroom)
 {
@@ -609,6 +633,18 @@ gossip_chatroom_get_status (GossipChatroom *chatroom)
 	return priv->status;
 }
 
+GossipChatroomStatus
+gossip_chatroom_get_occupants (GossipChatroom *chatroom)
+{
+	GossipChatroomPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), 0);
+
+	priv = GET_PRIV (chatroom);
+
+	return priv->occupants;
+}
+
 const gchar *
 gossip_chatroom_get_last_error (GossipChatroom *chatroom)
 {
@@ -620,22 +656,6 @@ gossip_chatroom_get_last_error (GossipChatroom *chatroom)
 	return priv->last_error;
 }
 
-GossipAccount *
-gossip_chatroom_get_account (GossipChatroom *chatroom)
-{
-	GossipChatroomPriv *priv;
-
-	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), NULL);
-
-	priv = GET_PRIV (chatroom);
-
-	if (!priv->account) {
-		return NULL;
-	}
-
-	return priv->account;
-}
-
 GossipChatroomContactInfo *
 gossip_chatroom_get_contact_info (GossipChatroom *chatroom,
 				  GossipContact  *contact)
@@ -650,6 +670,25 @@ gossip_chatroom_get_contact_info (GossipChatroom *chatroom,
 	return g_hash_table_lookup (priv->contacts, contact);
 }
 
+void
+gossip_chatroom_set_account (GossipChatroom *chatroom,
+			     GossipAccount  *account)
+{
+	GossipChatroomPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_CHATROOM (chatroom));
+	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
+
+	priv = GET_PRIV (chatroom);
+	if (priv->account) {
+		g_object_unref (priv->account);
+	}
+
+	priv->account = g_object_ref (account);
+
+	g_object_notify (G_OBJECT (chatroom), "account");
+}
+
 void
 gossip_chatroom_set_name (GossipChatroom *chatroom,
 			  const gchar    *name)
@@ -667,6 +706,40 @@ gossip_chatroom_set_name (GossipChatroom *chatroom,
 	g_object_notify (G_OBJECT (chatroom), "name");
 }
 
+void
+gossip_chatroom_set_description (GossipChatroom *chatroom,
+				 const gchar    *description)
+{
+	GossipChatroomPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_CHATROOM (chatroom));
+	g_return_if_fail (description != NULL);
+
+	priv = GET_PRIV (chatroom);
+
+	g_free (priv->description);
+	priv->description = g_strdup (description);
+
+	g_object_notify (G_OBJECT (chatroom), "description");
+}
+
+void
+gossip_chatroom_set_subject (GossipChatroom *chatroom,
+			     const gchar    *subject)
+{
+	GossipChatroomPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_CHATROOM (chatroom));
+	g_return_if_fail (subject != NULL);
+
+	priv = GET_PRIV (chatroom);
+
+	g_free (priv->subject);
+	priv->subject = g_strdup (subject);
+
+	g_object_notify (G_OBJECT (chatroom), "subject");
+}
+
 void
 gossip_chatroom_set_nick (GossipChatroom *chatroom,
 			  const gchar    *nick)
@@ -773,6 +846,31 @@ gossip_chatroom_set_favourite (GossipChatroom *chatroom,
 	g_object_notify (G_OBJECT (chatroom), "favourite");
 }
 
+static void
+chatroom_clear_contacts (GossipChatroom *chatroom)
+{
+	GossipChatroomPriv *priv;
+
+	priv = GET_PRIV (chatroom);
+
+	g_hash_table_remove_all (priv->contacts);
+}
+
+void
+gossip_chatroom_set_features (GossipChatroom         *chatroom,
+			      GossipChatroomFeature   features)
+{
+	GossipChatroomPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_CHATROOM (chatroom));
+
+	priv = GET_PRIV (chatroom);
+
+	priv->features = features;
+
+	g_object_notify (G_OBJECT (chatroom), "features");
+}
+
 void
 gossip_chatroom_set_status (GossipChatroom       *chatroom,
 			    GossipChatroomStatus  status)
@@ -785,9 +883,28 @@ gossip_chatroom_set_status (GossipChatroom       *chatroom,
 
 	priv->status = status;
 
+	if (status == GOSSIP_CHATROOM_STATUS_INACTIVE) {
+		chatroom_clear_contacts (chatroom);
+	}
+
 	g_object_notify (G_OBJECT (chatroom), "status");
 }
 
+void
+gossip_chatroom_set_occupants (GossipChatroom *chatroom,
+			       guint           occupants)
+{
+	GossipChatroomPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_CHATROOM (chatroom));
+
+	priv = GET_PRIV (chatroom);
+
+	priv->occupants = occupants;
+
+	g_object_notify (G_OBJECT (chatroom), "occupants");
+}
+
 void
 gossip_chatroom_set_last_error (GossipChatroom *chatroom,
 				const gchar    *last_error)
@@ -808,25 +925,6 @@ gossip_chatroom_set_last_error (GossipChatroom *chatroom,
 	g_object_notify (G_OBJECT (chatroom), "last_error");
 }
 
-void
-gossip_chatroom_set_account (GossipChatroom *chatroom,
-			     GossipAccount  *account)
-{
-	GossipChatroomPriv *priv;
-
-	g_return_if_fail (GOSSIP_IS_CHATROOM (chatroom));
-	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
-
-	priv = GET_PRIV (chatroom);
-	if (priv->account) {
-		g_object_unref (priv->account);
-	}
-
-	priv->account = g_object_ref (account);
-
-	g_object_notify (G_OBJECT (chatroom), "account");
-}
-
 void
 gossip_chatroom_set_contact_info (GossipChatroom            *chatroom,
 				  GossipContact             *contact,
@@ -934,16 +1032,6 @@ gossip_chatroom_equal_full (gconstpointer a,
 	return TRUE;
 }
 
-const gchar *
-gossip_chatroom_type_to_string (GossipChatroomType type)
-{
-	switch (type) {
-	case GOSSIP_CHATROOM_TYPE_NORMAL: return _("Normal");
-	}
-
-	return "";
-}
-
 const gchar *
 gossip_chatroom_status_to_string (GossipChatroomStatus status)
 {
@@ -1054,101 +1142,3 @@ gossip_chatroom_contact_left (GossipChatroom *chatroom,
 	}
 }
 
-/*
- * Chatroom invite functions
- */
-
-GossipChatroomInvite *
-gossip_chatroom_invite_ref (GossipChatroomInvite *invite)
-{
-	g_return_val_if_fail (invite != NULL, NULL);
-	g_return_val_if_fail (invite->ref_count > 0, NULL);
-
-	invite->ref_count++;
-
-	return invite;
-}
-
-void
-gossip_chatroom_invite_unref (GossipChatroomInvite *invite)
-{
-	g_return_if_fail (invite != NULL);
-	g_return_if_fail (invite->ref_count > 0);
-
-	invite->ref_count--;
-
-	if (invite->ref_count > 0) {
-		return;
-	}
-
-	if (invite->invitor) {
-		g_object_unref (invite->invitor);
-	}
-
-	g_free (invite->id);
-	g_free (invite->reason);
-}
-
-GType
-gossip_chatroom_invite_get_type (void)
-{
-	static GType type = 0;
-
-	if (type == 0) {
-		type = g_boxed_type_register_static
-			("GossipChatroomInvite",
-			 (GBoxedCopyFunc) gossip_chatroom_invite_ref,
-			 (GBoxedFreeFunc) gossip_chatroom_invite_unref);
-	}
-
-	return type;
-}
-
-GossipChatroomInvite *
-gossip_chatroom_invite_new (GossipContact *invitor,
-			    const gchar   *id,
-			    const gchar   *reason)
-{
-	GossipChatroomInvite *invite;
-
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (invitor), NULL);
-	g_return_val_if_fail (id != NULL, NULL);
-
-	invite = g_new0 (GossipChatroomInvite, 1);
-
-	invite->ref_count = 1;
-
-	invite->invitor = g_object_ref (invitor);
-	invite->id = g_strdup (id);
-
-	if (reason) {
-		invite->reason = g_strdup (reason);
-	}
-
-	return invite;
-}
-
-GossipContact *
-gossip_chatroom_invite_get_invitor (GossipChatroomInvite *invite)
-{
-	g_return_val_if_fail (invite != NULL, NULL);
-
-	return invite->invitor;
-}
-
-const gchar *
-gossip_chatroom_invite_get_id (GossipChatroomInvite *invite)
-{
-	g_return_val_if_fail (invite != NULL, NULL);
-
-	return invite->id;
-}
-
-const gchar *
-gossip_chatroom_invite_get_reason (GossipChatroomInvite *invite)
-{
-	g_return_val_if_fail (invite != NULL, NULL);
-
-	return invite->reason;
-}
-
