@@ -16,8 +16,6 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
- *
- * Authors: Martyn Russell <martyn@imendio.com>
  */
 
 #include "config.h"
@@ -31,7 +29,7 @@
 
 #include "gossip-session.h"
 #include "gossip-debug.h"
-#include "gossip-protocol.h"
+#include "gossip-jabber.h"
 #include "gossip-contact-manager.h"
 #include "gossip-account-manager.h"
 #include "gossip-private.h"
@@ -342,13 +340,13 @@ contact_manager_parse_contact (GossipContactManager *manager,
 
 	if (id && name) {
 		GossipContactManagerPriv *priv;
-		GossipProtocol           *protocol;
+		GossipJabber             *jabber;
 		GossipContact            *contact;
 
 		priv = GET_PRIV (manager);
 
-		protocol = gossip_session_get_protocol (priv->session, account); 
-		contact = gossip_protocol_new_contact (protocol, id, name);
+		jabber = gossip_session_get_protocol (priv->session, account); 
+		contact = gossip_jabber_new_contact (jabber, id, name);
 
 		gossip_contact_manager_add (manager, contact);
 	}
@@ -363,7 +361,7 @@ contact_manager_parse_self (GossipContactManager *manager,
 			    xmlNodePtr            node)
 {
 	GossipContactManagerPriv *priv;
-	GossipProtocol           *protocol;
+	GossipJabber             *jabber;
 	GossipContact            *contact;
 	const gchar              *id;
 	const gchar              *name;
@@ -376,8 +374,8 @@ contact_manager_parse_self (GossipContactManager *manager,
 
 	priv = GET_PRIV (manager);
 
-	protocol = gossip_session_get_protocol (priv->session, account); 
-	contact = gossip_protocol_get_own_contact (protocol);
+	jabber = gossip_session_get_protocol (priv->session, account); 
+	contact = gossip_jabber_get_own_contact (jabber);
 
 	id = gossip_contact_get_id (contact);
 	name = gossip_contact_get_name (contact);
@@ -459,13 +457,13 @@ contact_manager_file_parse (GossipContactManager *manager,
 		}
 
 		account = gossip_account_manager_find (account_manager, account_name);
-		xmlFree (account_name);
-
 		if (!account) {			
-			g_warning ("No GossipAccount found by name:'%s'", account_name);
+			gossip_debug (DEBUG_DOMAIN, "No GossipAccount found by name:'%s'", account_name);
 			continue;
 		}
 		
+		xmlFree (account_name);
+		
 		while (child) {
 			if (xmlNodeIsText (child)) {
 				child = child->next;
@@ -501,7 +499,9 @@ contact_manager_file_save (GossipContactManager *manager)
 	GossipAccount            *account;
 	gchar                    *directory;
 	gchar                    *filename;
+#ifndef G_OS_WIN32
 	mode_t                    old_mask;
+#endif /* G_OS_WIN32 */
 	xmlParserCtxtPtr          ctxt;
 	xmlDocPtr                 doc;
 	xmlNodePtr                root;
@@ -624,15 +624,16 @@ contact_manager_file_save (GossipContactManager *manager)
 
 	/* This is a self contact */
 	for (l = accounts; l; l = l->next) {
-		xmlNodePtr      node, child, p;
-		GossipProtocol *protocol;
-		GossipContact  *contact;
-		const gchar    *name; 
+		xmlNodePtr     node, child, p;
+		GossipJabber  *jabber;
+		GossipContact *contact;
+		const gchar   *name; 
 
 		account = l->data;
 		
-		protocol = gossip_session_get_protocol (priv->session, account);
-		contact = gossip_protocol_get_own_contact (protocol);
+		jabber = gossip_session_get_protocol (priv->session, account);
+		contact = gossip_jabber_get_own_contact (jabber);
+		
 		name = gossip_contact_get_name (contact);
 
 		node = g_hash_table_lookup (nodes, account);
@@ -726,11 +727,14 @@ contact_manager_file_save (GossipContactManager *manager)
 
 	gossip_debug (DEBUG_DOMAIN, "Saving file:'%s'", filename);
 
+#ifndef G_OS_WIN32
 	/* Set the umask to get the proper permissions when libxml saves the
 	 * file, but also change the permissions expiicitly in case the file
 	 * already exists.
 	 */
 	old_mask = umask (077);
+#endif
+
 	chmod (filename, 0600);
 
 	/* Make sure the XML is indented properly */
@@ -739,8 +743,10 @@ contact_manager_file_save (GossipContactManager *manager)
 
 	xmlSaveFormatFileEnc (filename, doc, "utf-8", 1);
 
+#ifndef G_OS_WIN32
 	/* Set back to original umask */
 	umask (old_mask);
+#endif /* G_OS_WIN32 */
 
 	xmlFreeDoc (doc);
 	xmlCleanupParser ();
