@@ -1,7 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2006 Imendio AB
- * Copyright (C) 2002-2003 Richard Hult <richard@imendio.com>
+ * Copyright (C) 2003-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,9 +16,12 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 #include <time.h>
@@ -27,12 +29,12 @@
 #include <regex.h>
 
 #include <glib/gi18n.h>
+
 #include <libxml/uri.h>
 
-#include "gossip-utils.h"
-#include "gossip-conf.h"
-#include "gossip-paths.h"
 #include "gossip-debug.h"
+#include "gossip-utils.h"
+#include "gossip-paths.h"
 
 #define DEBUG_DOMAIN "Utils"
 
@@ -219,89 +221,68 @@ gossip_xml_validate (xmlDoc      *doc,
 	return ret;
 }
 
-GType
-gossip_dbus_type_to_g_type (const gchar *dbus_type_string)
+xmlNodePtr
+gossip_xml_node_get_child (xmlNodePtr   node, 
+			   const gchar *child_name)
 {
-	if (dbus_type_string == NULL)
-		return G_TYPE_NONE;
+	xmlNodePtr l;
 
-	if (dbus_type_string[0] == 's') {
-		return G_TYPE_STRING;
-	}
-	else if (dbus_type_string[0] == 'b') {
-		return G_TYPE_BOOLEAN;
-	}
-	else if (dbus_type_string[0] == 'q') {
-		return G_TYPE_UINT;
-	}
-	else if (dbus_type_string[0] == 'n') {
-		return G_TYPE_INT;
-	}
+        g_return_val_if_fail (node != NULL, NULL);
+        g_return_val_if_fail (child_name != NULL, NULL);
 
-	g_assert_not_reached ();
-	return G_TYPE_NONE;
-}
-
-const gchar *
-gossip_g_type_to_dbus_type (GType g_type)
-{
-	switch (g_type)
-	{
-	case G_TYPE_STRING:
-		return "s";
-	case G_TYPE_BOOLEAN:
-		return "b";
-	case G_TYPE_UINT:
-		return "q";
-	case G_TYPE_INT:
-		return "n";
-	default:
-		g_assert_not_reached ();
+	for (l = node->children; l; l = l->next) {
+		if (l->name && strcmp (l->name, child_name) == 0) {
+			return l;
+		}
 	}
 
 	return NULL;
 }
 
-gchar *
-gossip_g_value_to_string (const GValue *value)
+xmlChar *
+gossip_xml_node_get_child_content (xmlNodePtr   node, 
+				   const gchar *child_name)
 {
-	gchar  *return_string = NULL;
-	GValue string_g_value = {0, };
+	xmlNodePtr l;
 
-	g_value_init (&string_g_value, G_TYPE_STRING);
-	g_value_transform (value, &string_g_value);
-	return_string = g_value_dup_string (&string_g_value);
-	g_value_unset (&string_g_value);
+        g_return_val_if_fail (node != NULL, NULL);
+        g_return_val_if_fail (child_name != NULL, NULL);
 
-	return return_string;
-}
-
-GValue *
-gossip_string_to_g_value (const gchar *str, GType type)
-{
-	GValue *g_value;
-
-	g_value = g_new0 (GValue, 1);
-	g_value_init (g_value, type);
-
-	switch (type)
-	{
-	case G_TYPE_BOOLEAN:
-		g_value_set_boolean (g_value, (str[0] == 'y' || str[0] == 'T'));
-		break;
-	case G_TYPE_UINT:
-		g_value_set_uint (g_value, atoi (str));
-		break;
-	case G_TYPE_INT:
-		g_value_set_int (g_value, atoi (str));
-		break;
-	case G_TYPE_STRING:
-		g_value_set_string (g_value, str);
-		break;
-	default:
-		g_assert_not_reached ();
+	l = gossip_xml_node_get_child (node, child_name);
+	if (l) {
+		return xmlNodeGetContent (l);
 	}
+		
+	return NULL;
+}
 
-	return g_value;
+xmlNodePtr
+gossip_xml_node_find_child_prop_value (xmlNodePtr   node, 
+				       const gchar *prop_name,
+				       const gchar *prop_value)
+{
+	xmlNodePtr l;
+	xmlNodePtr found = NULL;
+
+        g_return_val_if_fail (node != NULL, NULL);
+        g_return_val_if_fail (prop_name != NULL, NULL);
+        g_return_val_if_fail (prop_value != NULL, NULL);
+
+	for (l = node->children; l && !found; l = l->next) {
+		xmlChar *prop;
+
+		if (!xmlHasProp (l, prop_name)) {
+			continue;
+		}
+
+		prop = xmlGetProp (l, prop_name);
+		if (prop && strcmp (prop, prop_value) == 0) {
+			found = l;
+		}
+		
+		xmlFree (prop);
+	}
+		
+	return found;
 }
 
