@@ -23,6 +23,8 @@
 
 #include <glib-object.h>
 
+#include <gdk-pixbuf/gdk-pixbuf.h>
+
 #include "gossip-avatar.h"
 
 G_BEGIN_DECLS
@@ -47,33 +49,34 @@ struct _GossipVCardClass {
 
 GType         gossip_vcard_get_type        (void) G_GNUC_CONST;
 
-GossipVCard * gossip_vcard_new             (void);
+GossipVCard * gossip_vcard_new               (void);
 
-const gchar * gossip_vcard_get_name        (GossipVCard  *vcard);
-const gchar * gossip_vcard_get_nickname    (GossipVCard  *vcard);
-const gchar * gossip_vcard_get_birthday    (GossipVCard  *vcard);
-const gchar * gossip_vcard_get_email       (GossipVCard  *vcard);
-const gchar * gossip_vcard_get_url         (GossipVCard  *vcard);
-const gchar * gossip_vcard_get_country     (GossipVCard  *vcard);
-const gchar * gossip_vcard_get_description (GossipVCard  *vcard);
-GossipAvatar *gossip_vcard_get_avatar      (GossipVCard  *vcard);
+const gchar * gossip_vcard_get_name          (GossipVCard  *vcard);
+const gchar * gossip_vcard_get_nickname      (GossipVCard  *vcard);
+const gchar * gossip_vcard_get_birthday      (GossipVCard  *vcard);
+const gchar * gossip_vcard_get_email         (GossipVCard  *vcard);
+const gchar * gossip_vcard_get_url           (GossipVCard  *vcard);
+const gchar * gossip_vcard_get_country       (GossipVCard  *vcard);
+const gchar * gossip_vcard_get_description   (GossipVCard  *vcard);
+GossipAvatar *gossip_vcard_get_avatar        (GossipVCard  *vcard);
+GdkPixbuf *   gossip_vcard_create_avatar_pixbuf (GossipVCard *vcard);
 
-void          gossip_vcard_set_name        (GossipVCard  *vcard,
-					    const gchar  *name);
-void          gossip_vcard_set_nickname    (GossipVCard  *vcard,
-					    const gchar  *nickname);
-void          gossip_vcard_set_birthday    (GossipVCard  *vcard,
-					    const gchar  *birthday);
-void          gossip_vcard_set_email       (GossipVCard  *vcard,
-					    const gchar  *email);
-void          gossip_vcard_set_url         (GossipVCard  *vcard,
-					    const gchar  *url);
-void          gossip_vcard_set_country     (GossipVCard  *vcard,
-					    const gchar  *country);
-void          gossip_vcard_set_description (GossipVCard  *vcard,
-					    const gchar  *desc);
-void          gossip_vcard_set_avatar      (GossipVCard  *vcard,
-					    GossipAvatar *avatar);
+void          gossip_vcard_set_name          (GossipVCard  *vcard,
+					      const gchar  *name);
+void          gossip_vcard_set_nickname      (GossipVCard  *vcard,
+					      const gchar  *nickname);
+void          gossip_vcard_set_birthday      (GossipVCard  *vcard,
+					      const gchar  *birthday);
+void          gossip_vcard_set_email         (GossipVCard  *vcard,
+					      const gchar  *email);
+void          gossip_vcard_set_url           (GossipVCard  *vcard,
+					      const gchar  *url);
+void          gossip_vcard_set_country       (GossipVCard  *vcard,
+					      const gchar  *country);
+void          gossip_vcard_set_description   (GossipVCard  *vcard,
+					      const gchar  *desc);
+void          gossip_vcard_set_avatar        (GossipVCard  *vcard,
+					      GossipAvatar *avatar);
 
 G_END_DECLS
 
