@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005 Martyn Russell <mr@gnome.org>
+ * Copyright (C) 2005-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,15 +16,20 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
 #ifndef __GOSSIP_CHAT_INVITE_H__
 #define __GOSSIP_CHAT_INVITE_H__
 
 #include <libgossip/gossip-contact.h>
-#include <libgossip/gossip-chatroom-provider.h>
+
+G_BEGIN_DECLS
 
 void gossip_chat_invite_dialog_show (GossipContact    *contact,
 				     GossipChatroomId  id);
 
+G_END_DECLS
+
 #endif /* __GOSSIP_CHAT_INVITE_H__ */
