@@ -72,6 +72,15 @@ typedef struct {
 	GtkWidget        *scrolledwindow_chatrooms;
 	GossipChatView   *chatview_chatrooms;
 
+	GtkWidget        *vbox_links;
+	GtkWidget        *account_chooser_links;
+	GtkWidget        *entry_links;
+	GtkWidget        *button_links_open;
+	GtkWidget        *button_links_copy;
+	GtkWidget        *treeview_links;
+	GtkListStore     *treestore_links;
+	GtkTreeModel     *treemodel_links;
+
 	gchar            *last_find;
 
 	GossipLogManager *log_manager;
@@ -169,12 +178,32 @@ static void            log_window_entry_chatrooms_changed_cb          (GtkWidget
 								       GossipLogWindow  *window);
 static void            log_window_entry_chatrooms_activate_cb         (GtkWidget        *entry,
 								       GossipLogWindow  *window);
+
+/* Links */
+static void            log_window_links_changed_cb                    (GtkTreeSelection *selection,
+								       GossipLogWindow  *window);
+static void            log_window_links_populate                      (GossipLogWindow  *window);
+static void            log_window_links_setup                         (GossipLogWindow  *window);
+static gboolean        log_window_links_filter_func                   (GtkTreeModel     *model,
+								       GtkTreeIter      *iter,
+								       GossipLogWindow  *window);
+static void            log_window_links_accounts_changed_cb           (GtkWidget        *combobox,
+								       GossipLogWindow  *window);
+static void            log_window_entry_links_changed_cb              (GtkWidget        *entry,
+								       GossipLogWindow  *window);
+static void            log_window_button_links_open_clicked_cb        (GtkWidget        *widget,
+								       GossipLogWindow  *window);
+static void            log_window_button_links_copy_clicked_cb        (GtkWidget        *widget,
+								       GossipLogWindow  *window);
+
+/* Window */
 static void            log_window_destroy_cb                          (GtkWidget        *widget,
 								       GossipLogWindow  *window);
 
 enum {
 	COL_FIND_STATUS,
 	COL_FIND_ACCOUNT,
+	COL_FIND_ACCOUNT_NAME,
 	COL_FIND_CONTACT,
 	COL_FIND_CONTACT_NAME,
 	COL_FIND_DATE,
@@ -196,6 +225,18 @@ enum {
 	COL_CHATROOMS_COUNT
 };
 
+enum {
+	COL_LINKS_STATUS,
+	COL_LINKS_ACCOUNT,
+	COL_LINKS_ACCOUNT_NAME,
+	COL_LINKS_CONTACT,
+	COL_LINKS_CONTACT_NAME,
+	COL_LINKS_DATE,
+	COL_LINKS_DATE_READABLE,
+	COL_LINKS_URL,
+	COL_LINKS_COUNT
+};
+
 /*
  * Search code.
  */
@@ -371,6 +412,8 @@ log_window_find_populate (GossipLogWindow *window,
 	GtkListStore       *store;
 	GtkTreeIter         iter;
 
+	gossip_debug (DEBUG_DOMAIN, "Clearing search results treeview/textview");
+		
 	view = GTK_TREE_VIEW (window->treeview_find);
 	model = gtk_tree_view_get_model (view);
 	selection = gtk_tree_view_get_selection (view);
@@ -384,11 +427,14 @@ log_window_find_populate (GossipLogWindow *window,
 
 	if (G_STR_EMPTY (search_criteria)) {
 		/* Just clear the search. */
+		gossip_debug (DEBUG_DOMAIN, "No search results found");
 		return;
 	}
 
+	gossip_debug (DEBUG_DOMAIN, "Starting search...");
 	hits = gossip_log_search_new (window->log_manager, search_criteria);
 
+	gossip_debug (DEBUG_DOMAIN, "Adding %d hits", g_list_length (hits));
 	for (l = hits; l; l = l->next) {
 		const gchar *date;
 		gchar       *date_readable;
@@ -398,7 +444,7 @@ log_window_find_populate (GossipLogWindow *window,
 		account = gossip_log_search_hit_get_account (hit);
 		contact = gossip_log_search_hit_get_contact (hit);
 
-		/* Protect against invalid data (corrupt or old log files. */
+		/* Protect against invalid data (corrupt or old log files). */
 		if (!account || !contact) {
 			continue;
 		}
@@ -409,6 +455,7 @@ log_window_find_populate (GossipLogWindow *window,
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter,
 				    COL_FIND_ACCOUNT, account,
+				    COL_FIND_ACCOUNT_NAME, gossip_account_get_name (account),
 				    COL_FIND_CONTACT, contact,
 				    COL_FIND_CONTACT_NAME, gossip_contact_get_name (contact),
 				    COL_FIND_DATE, date,
@@ -442,8 +489,9 @@ log_window_find_setup (GossipLogWindow *window)
 	store = gtk_list_store_new (COL_FIND_COUNT,
 				    GDK_TYPE_PIXBUF,        /* account status */
 				    GOSSIP_TYPE_ACCOUNT,    /* account */
+				    G_TYPE_STRING,          /* account name */
 				    GOSSIP_TYPE_CONTACT,    /* contact */
-				    G_TYPE_STRING,          /* name */
+				    G_TYPE_STRING,          /* contact name */
 				    G_TYPE_STRING,          /* date */
 				    G_TYPE_STRING);         /* date_readable */
 
@@ -472,10 +520,10 @@ log_window_find_setup (GossipLogWindow *window)
 						 NULL);
 
 	gtk_tree_view_column_set_title (column, _("Account"));
-	gtk_tree_view_append_column (view, column);
-
+	gtk_tree_view_column_set_sort_column_id (column, COL_FIND_ACCOUNT_NAME);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_clickable (column, TRUE);
+	gtk_tree_view_append_column (view, column);
 
 	cell = gtk_cell_renderer_text_new ();
 	offset = gtk_tree_view_insert_column_with_attributes (view, -1, _("Conversation With"),
@@ -499,9 +547,7 @@ log_window_find_setup (GossipLogWindow *window)
 
 	/* Set up treeview properties */
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
-	gtk_tree_sortable_set_sort_column_id (sortable,
-					      COL_FIND_DATE,
-					      GTK_SORT_ASCENDING);
+	gtk_tree_sortable_set_sort_column_id (sortable, COL_FIND_DATE, GTK_SORT_ASCENDING);
 
 	/* Set up signals */
 	g_signal_connect (selection, "changed",
@@ -521,12 +567,17 @@ log_window_button_find_clicked_cb (GtkWidget       *widget,
 
 	/* Don't find the same crap again */
 	if (window->last_find && strcmp (window->last_find, str) == 0) {
+		gossip_debug (DEBUG_DOMAIN, 
+			      "Not searching for:'%s' in all log files "
+			      "(same as last search)", 
+			      str);
 		return;
 	}
 
 	g_free (window->last_find);
 	window->last_find = g_strdup (str);
 
+	gossip_debug (DEBUG_DOMAIN, "Searching for:'%s' in all log files", str);
 	log_window_find_populate (window, str);
 }
 
@@ -1748,6 +1799,330 @@ log_window_entry_chatrooms_activate_cb (GtkWidget       *entry,
 	}
 }
 
+/*
+ * Links
+ */
+static void
+log_window_links_changed_cb (GtkTreeSelection *selection,
+			     GossipLogWindow  *window)
+{
+	GtkTreeView   *view;
+	GtkTreeModel  *model;
+	GtkTreeIter    iter;
+
+	/* Get selected information */
+	view = GTK_TREE_VIEW (window->treeview_links);
+	model = gtk_tree_view_get_model (view);
+
+	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+		gtk_widget_set_sensitive (window->button_links_open, FALSE);
+		gtk_widget_set_sensitive (window->button_links_copy, FALSE);
+		return;
+	}
+
+	gtk_widget_set_sensitive (window->button_links_open, TRUE);
+	gtk_widget_set_sensitive (window->button_links_copy, TRUE);
+}
+
+static void
+log_window_links_populate (GossipLogWindow *window)
+{
+	GossipAccountChooser *account_chooser;
+	GossipAccount        *account;
+	GossipContact        *contact;
+
+	GList                *hits;
+	GList                *l;
+	GossipLogLinkHit     *hit;
+
+	GtkTreeView          *view;
+	GtkTreeSelection     *selection;
+	GtkListStore         *store;
+	GtkTreeIter           iter;
+
+	gossip_debug (DEBUG_DOMAIN, "Clearing link results treeview");
+
+	account_chooser = GOSSIP_ACCOUNT_CHOOSER (window->account_chooser_links);
+	account = gossip_account_chooser_get_account (account_chooser);
+
+	view = GTK_TREE_VIEW (window->treeview_links);
+	selection = gtk_tree_view_get_selection (view);
+	store = GTK_LIST_STORE (window->treestore_links);
+
+	gtk_list_store_clear (store);
+
+	gossip_debug (DEBUG_DOMAIN, "Getting links for %s%s...", 
+		      account ? "account " : "ALL accounts ",
+		      account ? gossip_account_get_name (account) : "");
+
+	hits = gossip_log_get_links (window->log_manager, account);
+
+	gossip_debug (DEBUG_DOMAIN, "Adding %d hits", g_list_length (hits));
+	for (l = hits; l; l = l->next) {
+		const gchar *url;
+		const gchar *date;
+		gchar       *date_readable;
+		const gchar *contact_name;
+
+		hit = l->data;
+
+		account = gossip_log_link_hit_get_account (hit);
+		contact = gossip_log_link_hit_get_contact (hit);
+
+		url = gossip_log_link_hit_get_url (hit);
+		date = gossip_log_link_hit_get_date (hit);
+		date_readable = gossip_log_get_date_readable (date);
+		
+		if (contact) {
+			contact_name = gossip_contact_get_name (contact);
+		} else {
+			contact_name = _("Unknown");
+		}
+		
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter,
+				    COL_LINKS_ACCOUNT, account,
+				    COL_LINKS_ACCOUNT_NAME, gossip_account_get_name (account),
+				    COL_LINKS_CONTACT, contact,
+				    COL_LINKS_CONTACT_NAME, contact_name,
+				    COL_LINKS_DATE, date,
+				    COL_LINKS_DATE_READABLE, date_readable,
+				    COL_LINKS_URL, url,
+				    -1);
+
+		g_free (date_readable);
+	}
+
+	if (hits) {
+		gossip_log_links_free (hits);
+	}
+}
+
+static void
+log_window_links_setup (GossipLogWindow *window)
+{
+	GtkTreeView       *view;
+	GtkTreeModel      *model;
+	GtkTreeSelection  *selection;
+	GtkTreeSortable   *sortable;
+	GtkTreeViewColumn *column;
+	GtkCellRenderer   *cell;
+	gint               offset;
+
+	if (window->treestore_links) {
+		g_object_unref (window->treestore_links);
+	}
+
+	if (window->treemodel_links) {
+		g_object_unref (window->treemodel_links);
+	}
+
+	view = GTK_TREE_VIEW (window->treeview_links);
+	selection = gtk_tree_view_get_selection (view);
+
+	/* New store */
+	window->treestore_links = gtk_list_store_new (COL_LINKS_COUNT,
+						      GDK_TYPE_PIXBUF,        /* account status */
+						      GOSSIP_TYPE_ACCOUNT,    /* account */
+						      G_TYPE_STRING,          /* account name */
+						      GOSSIP_TYPE_CONTACT,    /* contact */
+						      G_TYPE_STRING,          /* contact name */
+						      G_TYPE_STRING,          /* date */
+						      G_TYPE_STRING,          /* date_readable */
+						      G_TYPE_STRING);         /* url */
+
+	model = GTK_TREE_MODEL (window->treestore_links);
+	sortable = GTK_TREE_SORTABLE (window->treestore_links);
+
+	gtk_tree_view_set_model (view, model);
+
+	/* Create filter */
+	window->treemodel_links = gtk_tree_model_filter_new (model, NULL);
+
+	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (window->treemodel_links),
+						(GtkTreeModelFilterVisibleFunc)
+						log_window_links_filter_func,
+						window, NULL);
+
+	gtk_tree_view_set_model (view, window->treemodel_links);
+
+	/* Account */
+	column = gtk_tree_view_column_new ();
+
+	cell = gtk_cell_renderer_pixbuf_new ();
+	gtk_tree_view_column_pack_start (column, cell, FALSE);
+	gtk_tree_view_column_set_cell_data_func (column, cell,
+						 (GtkTreeCellDataFunc)
+						 log_window_find_pixbuf_data_func,
+						 window,
+						 NULL);
+
+	cell = gtk_cell_renderer_text_new ();
+	gtk_tree_view_column_pack_start (column, cell, TRUE);
+	gtk_tree_view_column_set_cell_data_func (column, cell,
+						 (GtkTreeCellDataFunc)
+						 log_window_find_text_data_func,
+						 window,
+						 NULL);
+
+	gtk_tree_view_column_set_title (column, _("Account"));
+	gtk_tree_view_column_set_sort_column_id (column, COL_LINKS_ACCOUNT_NAME);
+	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_clickable (column, TRUE);
+	gtk_tree_view_append_column (view, column);
+
+	/* Who */
+	cell = gtk_cell_renderer_text_new ();
+	g_object_set (cell, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+	offset = gtk_tree_view_insert_column_with_attributes (view, -1, _("Conversation With"),
+							      cell, "text", COL_LINKS_CONTACT_NAME,
+							      NULL);
+
+	column = gtk_tree_view_get_column (view, offset - 1);
+	gtk_tree_view_column_set_sort_column_id (column, COL_LINKS_CONTACT_NAME);
+	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_clickable (column, TRUE);
+
+	/* Date */
+	cell = gtk_cell_renderer_text_new ();
+	offset = gtk_tree_view_insert_column_with_attributes (view, -1, _("Date"),
+							      cell, "text", COL_LINKS_DATE_READABLE,
+							      NULL);
+
+	column = gtk_tree_view_get_column (view, offset - 1);
+	gtk_tree_view_column_set_sort_column_id (column, COL_LINKS_DATE);
+	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_clickable (column, TRUE);
+
+	/* Link */
+	cell = gtk_cell_renderer_text_new ();
+	g_object_set (cell, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+	offset = gtk_tree_view_insert_column_with_attributes (view, -1, _("Link"),
+							      cell, "text", COL_LINKS_URL,
+							      NULL);
+
+	column = gtk_tree_view_get_column (view, offset - 1);
+	gtk_tree_view_column_set_sort_column_id (column, COL_LINKS_URL);
+	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_clickable (column, TRUE);
+
+	/* Set up treeview properties */
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+	gtk_tree_sortable_set_sort_column_id (sortable, COL_LINKS_DATE, GTK_SORT_ASCENDING);
+
+	/* Set up signals */
+	g_signal_connect (selection, "changed",
+			  G_CALLBACK (log_window_links_changed_cb),
+			  window);
+}
+
+static gboolean
+log_window_links_filter_func (GtkTreeModel    *model,
+			      GtkTreeIter     *iter,
+			      GossipLogWindow *window)
+{
+	const gchar *filter;
+	gchar       *filter_casefold;
+	gchar       *str;
+	gchar       *str_casefold;
+	gboolean     visible = FALSE;
+	gint         col;
+	gint         i;
+
+	filter = gtk_entry_get_text (GTK_ENTRY (window->entry_links));
+	if (G_STR_EMPTY (filter)) {
+		return TRUE;
+	}
+
+	filter_casefold = g_utf8_casefold (filter, -1);
+
+	for (i = 0, col = 0; i < 3 && !visible; i++) {
+		switch (i) {
+		case 0: col = COL_LINKS_ACCOUNT_NAME; break;
+		case 1: col = COL_LINKS_CONTACT_NAME; break;
+		case 2: col = COL_LINKS_URL;          break;
+		default:   			      continue;
+		}
+
+		gtk_tree_model_get (model, iter, col, &str, -1);
+		if (G_STR_EMPTY (str)) {
+			continue;
+		}
+		
+		str_casefold = g_utf8_casefold (str, -1);
+		visible |= strstr (str_casefold, filter_casefold) != NULL;
+		g_free (str_casefold);
+	}
+	
+	g_free (filter_casefold);
+
+	return visible;
+}
+
+static void
+log_window_links_accounts_changed_cb (GtkWidget       *combobox,
+				      GossipLogWindow *window)
+{
+	log_window_links_populate (window);
+}
+
+static void
+log_window_entry_links_changed_cb (GtkWidget       *entry,
+				   GossipLogWindow *window)
+{
+	gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (window->treemodel_links));
+}
+
+static void
+log_window_button_links_open_clicked_cb (GtkWidget       *widget,
+					 GossipLogWindow *window)
+{
+	GtkTreeView      *view;
+	GtkTreeModel     *model;
+	GtkTreeSelection *selection;
+	GtkTreeIter       iter;
+	gchar            *url;
+
+	/* Get selected information */
+	view = GTK_TREE_VIEW (window->treeview_links);
+	model = gtk_tree_view_get_model (view);
+	selection = gtk_tree_view_get_selection (view);
+
+	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+		return;
+	}
+	
+	gtk_tree_model_get (model, &iter, COL_LINKS_URL, &url, -1);
+	gossip_url_show (url);
+	g_free (url);
+}
+
+static void
+log_window_button_links_copy_clicked_cb (GtkWidget       *widget,
+					 GossipLogWindow *window)
+{
+	GtkTreeView      *view;
+	GtkTreeModel     *model;
+	GtkTreeSelection *selection;
+	GtkTreeIter       iter;
+	GtkClipboard     *clipboard;
+	gchar            *url;
+
+	/* Get selected information */
+	view = GTK_TREE_VIEW (window->treeview_links);
+	model = gtk_tree_view_get_model (view);
+	selection = gtk_tree_view_get_selection (view);
+
+	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+		return;
+	}
+	
+	gtk_tree_model_get (model, &iter, COL_LINKS_URL, &url, -1);
+	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text (clipboard, url, -1);
+	g_free (url);
+}
+
 /*
  * Other window callbacks
  */
@@ -1757,6 +2132,9 @@ log_window_destroy_cb (GtkWidget       *widget,
 {
 	g_free (window->last_find);
 
+	g_object_unref (window->treestore_links);
+	g_object_unref (window->treemodel_links);
+
 	gossip_log_handler_remove
 		(window->log_manager,
 		 (GossipLogMessageFunc) log_window_contacts_new_message_cb);
@@ -1777,8 +2155,6 @@ gossip_log_window_show (GossipContact  *contact,
 	GossipSession          *session;
 	GossipLogManager       *log_manager;
 	GossipAccountChooser   *account_chooser;
-	GList                  *accounts;
-	gint                    account_num;
 	GladeXML               *glade;
 
 	if (window) {
@@ -1821,6 +2197,11 @@ gossip_log_window_show (GossipContact  *contact,
 				       "vbox_chatrooms", &window->vbox_chatrooms,
 				       "treeview_chatrooms", &window->treeview_chatrooms,
 				       "scrolledwindow_chatrooms", &window->scrolledwindow_chatrooms,
+				       "vbox_links", &window->vbox_links,
+				       "entry_links", &window->entry_links,
+				       "button_links_open", &window->button_links_open,
+				       "button_links_copy", &window->button_links_copy,
+				       "treeview_links", &window->treeview_links,
 				       NULL);
 	gossip_glade_connect (glade,
 			      window,
@@ -1833,6 +2214,9 @@ gossip_log_window_show (GossipContact  *contact,
 			      "entry_contacts", "activate", log_window_entry_contacts_activate_cb,
 			      "entry_chatrooms", "changed", log_window_entry_chatrooms_changed_cb,
 			      "entry_chatrooms", "activate", log_window_entry_chatrooms_activate_cb,
+			      "entry_links", "changed", log_window_entry_links_changed_cb,
+			      "button_links_open", "clicked", log_window_button_links_open_clicked_cb,
+			      "button_links_copy", "clicked", log_window_button_links_copy_clicked_cb,
 			      NULL);
 
 	g_object_unref (glade);
@@ -1873,17 +2257,15 @@ gossip_log_window_show (GossipContact  *contact,
 			   GTK_WIDGET (window->chatview_chatrooms));
 	gtk_widget_show (GTK_WIDGET (window->chatview_chatrooms));
 
-	/* Account chooser for contacts & chat rooms */
+	/* Account chooser for contacts/chat rooms/links */
 	session = gossip_app_get_session ();
 
 	window->account_chooser_contacts = gossip_account_chooser_new (session);
 	account_chooser = GOSSIP_ACCOUNT_CHOOSER (window->account_chooser_contacts);
 	gossip_account_chooser_set_can_select_all (account_chooser, TRUE);
-
 	gtk_box_pack_start (GTK_BOX (window->vbox_contacts),
 			    window->account_chooser_contacts,
 			    FALSE, TRUE, 0);
-
 	g_signal_connect (window->account_chooser_contacts, "changed",
 			  G_CALLBACK (log_window_contacts_accounts_changed_cb),
 			  window);
@@ -1891,31 +2273,40 @@ gossip_log_window_show (GossipContact  *contact,
 	window->account_chooser_chatrooms = gossip_account_chooser_new (session);
 	account_chooser = GOSSIP_ACCOUNT_CHOOSER (window->account_chooser_chatrooms);
 	gossip_account_chooser_set_can_select_all (account_chooser, TRUE);
-
 	gtk_box_pack_start (GTK_BOX (window->vbox_chatrooms),
 			    window->account_chooser_chatrooms,
 			    FALSE, TRUE, 0);
-
 	g_signal_connect (window->account_chooser_chatrooms, "changed",
 			  G_CALLBACK (log_window_chatrooms_accounts_changed_cb),
 			  window);
+
+	window->account_chooser_links = gossip_account_chooser_new (session);
+	account_chooser = GOSSIP_ACCOUNT_CHOOSER (window->account_chooser_links);
+	gossip_account_chooser_set_has_all_option (account_chooser, TRUE);
+	gossip_account_chooser_set_can_select_all (account_chooser, TRUE);
+	gossip_account_chooser_set_account (account_chooser, NULL);
+	gtk_box_pack_start (GTK_BOX (window->vbox_links),
+			    window->account_chooser_links,
+			    FALSE, TRUE, 0);
+	g_signal_connect (window->account_chooser_links, "changed",
+			  G_CALLBACK (log_window_links_accounts_changed_cb),
+			  window);
+	
 	/* Populate */
-	accounts = gossip_session_get_accounts (session);
-	account_num = g_list_length (accounts);
-
-	g_list_foreach (accounts, (GFunc)g_object_unref, NULL);
-	g_list_free (accounts);
-
-	if (account_num > 1) {
+	if (gossip_account_chooser_get_count (account_chooser) > 1) {
 		gtk_widget_show (window->vbox_contacts);
 		gtk_widget_show (window->account_chooser_contacts);
 		gtk_widget_show (window->vbox_chatrooms);
 		gtk_widget_show (window->account_chooser_chatrooms);
+		gtk_widget_show (window->vbox_links);
+		gtk_widget_show (window->account_chooser_links);
 	} else {
 		gtk_widget_hide (window->vbox_contacts);
 		gtk_widget_hide (window->account_chooser_contacts);
 		gtk_widget_hide (window->vbox_chatrooms);
 		gtk_widget_hide (window->account_chooser_chatrooms);
+		gtk_widget_hide (window->vbox_links);
+		gtk_widget_hide (window->account_chooser_links);
 	}
 
 	/* Search List */
@@ -1929,6 +2320,10 @@ gossip_log_window_show (GossipContact  *contact,
 	log_window_chatrooms_setup (window);
 	log_window_chatrooms_populate (window);
 
+	/* Links List */
+	log_window_links_setup (window);
+	log_window_links_populate (window);
+
 	/* Select contact or chatroom */
 	if (contact) {
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (window->notebook), 1);
@@ -1936,6 +2331,8 @@ gossip_log_window_show (GossipContact  *contact,
 	} else if (chatroom) {
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (window->notebook), 2);
 		log_window_chatrooms_set_selected (window, chatroom);
+	} else {
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (window->notebook), 0);
 	}
 
 	gtk_widget_show (window->window);
